/*
LcdTask.c

Copyright 1998-2001 Larry Barello

Author: Larry Barello
        larry@barello.net
*/

#define ENABLE_BIT_DEFINITIONS
#include "avrx-io.h"
#include <progmem.h>
#include "avrx.h"
#include "hardware.h"
#include "MotorTask.h"
#include "lcdio.h"
#include "LcdTask.h"
#include "adcdriver.h"
#include "mob_eeprom.h"

TimerControlBlock LcdTimer;

static char FLASH sSplash1[] = " Modular OSMC Brain ";
static char FLASH sSplash2[] = "      >>MOB<<";
static char FLASH sSplash3[] = "  The OSMC Project";
static char FLASH sSplash4[] = "v0.54       05/12/02";

static char FLASH sPmode[]  = "PWM Mode: ";
static char FLASH sSignMag[]= "SignMag ";
static char FLASH sLocked[] = "LockAnt ";
static char FLASH sOutput[] = "Output: ";
static char FLASH sFlip[] = "Detect Flip: ";
static char FLASH sDisabled[]="Disabled ";
static char FLASH sEnabled[]= "Enabled ";
static char FLASH sYes[] = "Yes ";
static char FLASH sNo[] = "No ";
static char FLASH sRate[]   = "PWM Rate: ";
static char FLASH s2k[]     = " 2kHz ";
static char FLASH s4k[]     = " 4kHz ";
static char FLASH s8k[]     = " 8kHz ";
static char FLASH s16k[]    = "16kHz ";
static char FLASH s32k[]    = "32kHz ";

static char FLASH sMixMode[]= "Mixing Mode: ";
static char FLASH sMixed[]  = "Mix ";
static char FLASH sNormal[] = "Norm";
static char FLASH sExpo[]	= "Expo";

static char FLASH sVl[]     = "LBat:";
static char FLASH sVr[]     = "RBat:";
static char FLASH sMl[]     = "L:";
static char FLASH sMr[]     = "R:";
static char FLASH sMa[]		= "A:";
static char FLASH sFl[]		= "Flip";
static char FLASH sFln[]	= "    ";
static char FLASH sOK[]		= "OK";
static char FLASH sXX[]		= "XX";

static char FLASH sCommit[] = "Store Changes? ";
static char FLASH sDone[]   = "Done!";
static char FLASH sDebug1[] = "# in out Ok Volt";
static char FLASH sHalt[]   = "Fatal Error ";
static char FLASH sDeadband[]="DeadBand: ";
static char FLASH sCount[]     = " Counts";
static char FLASH sReset[]  = "EEPROM Corrupt! ";
static char FLASH sReset1[] = "Any key resets";
static char FLASH sLed[]    = "Led Mode: ";
static char FLASH sSlew[]   = "Slew Rate: 0.";
static char FLASH sSlew1[]  = " sec";
static char FLASH sSlew2[]	= "Slew:.";
static char FLASH sBrake[]  = "Brake Amount: ";
static char FLASH sBrk[]	= "Brake:";
static char FLASH sBrake1[] = "0%";


void lcd_putSP(void)
{
    lcd_put_char(' ');
}

void lcd_PutTenths(int val)
{
    char i;
    for (i = 0; val >=100; i++)
        val -= 100;
    lcd_PutHexChar(i);
    for (i = 0; val >=10; i++)
        val -= 10;
    lcd_PutHexChar(i);
    lcd_put_char('.');
    lcd_PutHexChar(val);
}

void lcd_PutDecimal(char val)   // 0-99
{
    char i;
    for (i = 0; val >=10; i++)
        val -= 10;
    lcd_PutHexChar(i);
    lcd_PutHexChar(val);
}

void lcd_PutDecWord(int val)
{
    char i;
    
    if (val < 0)
    {
        val = -val;
        lcd_put_char('-');
    }
    else
        lcd_putSP();
        
    for (i = 0; val >=10000; i++)
        val -= 10000;
    lcd_PutHexChar(i);
    for (i = 0; val >=1000; i++)
        val -= 1000;
    lcd_PutHexChar(i);
    lcd_PutTenths(val);
}

void HaltError(char ploc)
{
    lcd_wr_inst(LCD_CLEAR);
    lcd_wr_inst(LCD_HOME);
    lcd_putstr(sHalt);
    lcd_PutHexByte(ploc);
    AvrXHalt();
}

/*+---------------------------------------------------------

    void PrintPWM_{Rate, Mode}(void)
    
Helper for menus
*/
void PrintPWM_Rate(void)
{
    switch (PWM_Rate)
   {
        default:
            PWM_Rate = 0;       // Handle overflow (wrap)
        case PWM2K:
            lcd_putstr(s2k);
            break;
        case PWM4K:
            lcd_putstr(s4k);
            break;
        case PWM16K:
            lcd_putstr(s16k);
            break;
        case -1:                // Handle underflow (wrap)
            PWM_Rate = PWM32K;
        case PWM32K:
            lcd_putstr(s32k);
            break;
    }
}

void PrintPWM_Mode(void)
{
    switch (PWM_Mode)
    {
        default:
            PWM_Mode = 0;
        case LOCKED_ANTIPHASE:
            lcd_putstr(sLocked);
            break;
        case SIGN_MAGNITUDE:
            lcd_putstr(sSignMag);
            break;
    }
}

void PrintMixedMode(void)
{
    switch(MixedMode)
    {
        default:
            MixedMode = 0;
        case MIXED:
            lcd_putstr(sMixed);
            break;
        case STRAIGHT:
            lcd_putstr(sNormal);
            break;
		//case EXPO:
		//	lcd_putstr(sExpo);
		//	break;
    }
}


void PrintMotorSpeed(pRCcontrolBlock p)
{
    //lcd_putSP();
    lcd_PutHexByte(p->Drive);	
    //lcd_PutHexByte(0x3F);	
}


void PrintDebugInfo(pRCcontrolBlock p)
{
	lcd_putSP();
   lcd_PutHexByte(p->rcpulse);
    lcd_putSP();
    lcd_PutHexByte(p->Drive);
    lcd_putSP();
    lcd_putSP();
    if (!p->valid)
        lcd_put_char('N');
    else
        lcd_put_char('Y');
    lcd_putSP();
    lcd_putSP();
    lcd_putSP();
}

// Helper to read analog channel, convert to a reasonable number

char GetVoltage(char channel)
{
    return ((ADCReadChannel(channel)*VFULLSCALE)/0x3FF);
}

/*
    LcdTask updates the LCD display 5 times/second.
    Run at low priority since it is slow

*/
AVRX_GCC_TASKDEF(LcdTask, 24, 3)
{
    char    led         = 0,
            option      = 0,
            LCDmode     = 0,
            PrevLCDmode = 1,
            StatusMode  = 0,
            sw_current  = 0,
            sw_stable   = 0,
            sw_changed,
            sw_tmp;
            
    ADC_Init();
    
    lcd_4_init();
    lcd_wr_inst(LCD_FUNC | LCD_N);
    lcd_wr_inst(LCD_CLEAR);
    lcd_wr_inst(LCD_HOME);
    lcd_wr_inst(LCD_DISPLAY | LCD_D ); // | LCD_C

    // Splash screen display
    // of controller version info
   RW_LCD();
	lcd_wr_inst(LCD_CLEAR);
	lcd_wr_inst(LCD_HOME);
	lcd_putstr(sSplash1);
	lcd_setcursor(LCD_LINE2);
	lcd_putstr(sSplash2);
	lcd_setcursor(LCD_LINE3);
	lcd_putstr(sSplash3);
	lcd_setcursor(LCD_LINE4);
	lcd_putstr(sSplash4);
	AvrXDelay(&LcdTimer, MILLISECOND(2000));// Delay 3 seconds before starting.

    while(1)
    {
        AvrXStartTimer(&LcdTimer, MILLISECOND(50));

        // Update display on every fourth tick (200ms)
        
        if ((++led & 0x03) == 3)
        {
            RW_LCD();
            if ((LCDmode != PrevLCDmode) || option)
            {
                lcd_wr_inst(LCD_CLEAR);      // Don't clear screen unless mode changes
                PrevLCDmode = LCDmode;
            }
            lcd_wr_inst(LCD_HOME);

            if (EEpromResetFlag == TRUE)
            {
                lcd_putstr(sReset);
                lcd_setcursor(LCD_LINE2);
                lcd_putstr(sReset1);
                if (option)
                {
                    lcd_putstr(sDone);
                    option = 0;
                    ResetEEPROM();
                }
                continue;
            }
                
            // Note: Order of switch statements has no relationship
            // to the enumeration order: see LcdTask.h

            switch(LCDmode)
            {
                case D_COMMIT:
                    lcd_putstr(sCommit);
                    if (option)
                    {
                        lcd_putstr(sDone);
                        WriteEEPROMInitModes();
						LCDmode = D_STATUS;
                    }
                    break;
                case D_DEADBAND:
                    lcd_putstr(sDeadband);
                    DeadBand += option;
                    if (DeadBand > DEADBAND_RANGE) DeadBand = 0;
                    if (DeadBand < 0) DeadBand = DEADBAND_RANGE;
                    lcd_PutHexByte(DeadBand);
                    lcd_putstr(sCount);
                    break;
                case D_MODE:
                    lcd_putstr(sPmode);
                    if (option)
                    {
                        PWM_Mode++;
                    }
                    PrintPWM_Mode();
                    break;
                case D_RATE:
                    lcd_putstr(sRate);
                    PWM_Rate += option;
                    PrintPWM_Rate();
                    break;
                case D_MIX:
                    lcd_putstr(sMixMode);
                    if (option)
                    {
                        MixedMode++;
                    }
                    PrintMixedMode();
                    break;
                case D_STATUS:
                    if (option)
                    {
                        StatusMode++;
                    }
                    switch(StatusMode)
                    {
                        default:
                            StatusMode = 0;
                        case STATUS:
                            PrintPWM_Mode();
                            PrintPWM_Rate();
                            PrintMixedMode();
                           
						   lcd_setcursor(LCD_LINE2);
									lcd_putstr(sMl);
									PrintMotorSpeed(&RC0);
                           lcd_putSP();
									lcd_putstr(sMr);
									PrintMotorSpeed(&RC1);
                           lcd_putSP();
									lcd_putstr(sMa);
									PrintMotorSpeed(&RC2);
                           lcd_putSP();
						   if (Flipped)
						   {
							   lcd_putstr(sFl);
						   }
						   else 
						   {
							   lcd_putstr(sFln);
						   }

						   lcd_setcursor(LCD_LINE3);
                           lcd_putstr(sVl);
                           lcd_PutDecimal(GetVoltage(ADCLeftBattery));
                           lcd_putSP();
						   lcd_putstr(sVr);
						   lcd_PutDecimal(GetVoltage(ADCRightBattery));

						   lcd_setcursor(LCD_LINE4);
							lcd_putstr(sBrk);
							lcd_PutDecimal(BrakeMode);
							lcd_putstr(sBrake1);
							lcd_putSP();
							lcd_putstr(sSlew2);
							lcd_PutDecimal(__lpm_inline((unsigned)&SlewTable[(int)SlewRate].time));
							//lcd_putstr(sSlew1);
							lcd_put_char('s');

                           break;
                        case DEBUG:
                            lcd_putstr(sDebug1);
                            lcd_setcursor(LCD_LINE2);
                            lcd_put_char('R');
                            PrintDebugInfo(&RC0);
                            lcd_PutDecimal(GetVoltage(ADCRightBattery));
                            lcd_setcursor(LCD_LINE3);
                            lcd_put_char('L');
                            PrintDebugInfo(&RC1);
                            lcd_PutDecimal(GetVoltage(ADCLeftBattery));
                            lcd_setcursor(LCD_LINE4);
                            lcd_put_char('A');
                            PrintDebugInfo(&RC2);
                            break;
                    }
                    break;
				case D_FLIP:
					if (option)
					{
						Flip_Mode = !Flip_Mode; 
					}
					lcd_putstr(sFlip);
					if (Flip_Mode)
					{
						lcd_putstr(sYes);
					}
					else 
					{
						lcd_putstr(sNo);
					}
					break;
                case D_DISABLE:
                    if (option)
                    {
                        fDisable.lcd_disable = !fDisable.lcd_disable;
                    }
                    lcd_putstr(sOutput);
                    if (fDisable.lcd_disable == TRUE)
                    {
                        lcd_putstr(sDisabled);
                    }
                    else
                    {
                        lcd_putstr(sEnabled);
                    }
                    break;

                case D_BRAKE:
                    BrakeMode += (option *2);   // 0,2,4,6,8,10 - compared to PulseFailTimer
                    if (BrakeMode > 10)         // to PWM the disable line & get variable braking.
                        BrakeMode = 0;
                    if (BrakeMode < 0)
                        BrakeMode = 10;

                    lcd_putstr(sBrake);
                    lcd_PutDecimal(BrakeMode);
                    lcd_putstr(sBrake1);

                    break;

                case D_SLEW:
                    SlewRate += option;       // 0, 5, 10, 15, 20 & 25 - Not sure how this is used
                    if (SlewRate < 0)           // SlewTable has 5 entries
                        SlewRate = 5;
                    if (SlewRate > 5)
                        SlewRate = 0;

                    lcd_putstr(sSlew);
                    lcd_PutDecimal(__lpm_inline((unsigned)&SlewTable[(int)SlewRate].time));
                    lcd_putstr(sSlew1);

                    break;

                default:
                    LCDmode = 0;      // Reset mode
                    break;
                case -1:
                    LCDmode = D_UPPER;
            }
            option = 0;                 // Reset
        }
        
        // Evaluate Switch Input (need second RD_BUTTONS()
        // to give switch buffer a cycle to change directions.

        RD_BUTTONS();   // Buffer Enabled
        RD_BUTTONS();   // Buffer drives lines

        // Debounce switch input
        
        sw_tmp      = sw_current;
        sw_current  = LCD_SWITCHES();
        sw_tmp      = sw_tmp ^ sw_current;
        sw_changed  = (sw_current ^ sw_stable) & ~sw_tmp;
        sw_stable   = (sw_stable & sw_tmp) | (sw_current & ~sw_tmp);

        // Now parse the debounced switch information
        // Switches recently "opened" will be stable & changed.

		// N.B. UP/DOWN and LEFT/RIGHT are reversed in the hardware
		// LEFT/RIGHT buttons actually change mode etc.
		// C.B. 12/18/01

        switch(sw_stable & sw_changed)
        {
            case BV(SW_LEFT):
                --option;
                break;
            case BV(SW_RIGHT):
                ++option;
                break;
            case BV(SW_UP):
                --LCDmode;
                break;
            case BV(SW_DOWN):
                ++LCDmode;
                break;
        }

        AvrXWaitTimer(&LcdTimer);
    }   // end while()
}

